// -*- C++ -*-
/*!
 * @file DataOutPort.h
 * @brief Base class of OutPort
 * @date $Date: 2008-01-13 15:06:58 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: DataOutPort.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef DataOutPort_h
#define DataOutPort_h


#include <rtm/PortBase.h>
#include <rtm/BufferBase.h>
#include <rtm/OutPortCorbaProvider.h>
#include <rtm/InPortCorbaConsumer.h>
#include <rtm/OutPort.h>
#include <rtm/NVUtil.h>
#include <rtm/PublisherFactory.h>
#include <rtm/Properties.h>

#ifdef RTC_SOCK_DATAPORT
#include <rtm/OutPortTcpSockProvider.h>
#include <rtm/InPortTcpSockConsumer.h>
#endif

namespace RTC
{
  /*!
   * @if jp
   * @class DataOutPort
   * @brief Outort p Port
   *
   * f[^o̓|[g̎NXB
   * 
   * OutPort  connect() ł͈ȉ̃V[PXŏsB
   *
   * 1. OutPort Ɋ֘A connector ̐уZbg
   *
   * 2. InPortɊ֘A connector ̎擾
   *  - ConnectorProfile::properties["dataport.corba_any.inport_ref"]
   *    OutPortAny ̃IuWFNgt@Xݒ肳ĂꍇA
   *    t@X擾ConsumerIuWFNgɃZbgB
   *    t@XZbgĂȂΖČpB
   *    (OutPortconnect() ĂяõGg|Cg̏ꍇ́A
   *    InPort̃IuWFNgt@X̓ZbgĂȂ͂łB)
   * 3. PortBase::connect() R[
   *    Port̐ڑ̊{sB
   * 4. L2.InPort̃t@X擾łȂ΁AēxInPort
   *    ֘A connector 擾B
   *
   * 5. ConnectorProfile::properties ŗ^ꂽ񂩂A
   *    OutPort̏sB
   *
   * - [dataport.interface_type]
   * -- CORBA_Any ̏ꍇ: 
   *    InPortAny ʂăf[^B
   *    ConnectorProfile::properties["dataport.corba_any.inport_ref"]
   *    InPortAny ̃IuWFNgt@XZbgB
   * -- RawTCP ̏ꍇ: Raw TCP socket ʂăf[^B
   *    ConnectorProfile::properties["dataport.raw_tcp.server_addr"]
   *    InPort̃T[oAhXZbgB
   *
   * - [dataport.dataflow_type]
   * -- Push̏ꍇ: Subscriber𐶐BSubscriber̃^Cv́A
   *    dataport.subscription_type ɐݒ肳ĂB
   * -- Pull̏ꍇ: InPortf[^Pull^Ŏ擾邽߁A
   *    ɉKvB
   *
   * - [dataport.subscription_type]
   * -- Oncȅꍇ: SubscriberOnce𐶐B
   * -- New̏ꍇ: SubscriberNew𐶐B
   * -- Periodic̏ꍇ: SubscriberPeriodic𐶐B
   *
   * - [dataport.push_interval]
   * -- dataport.subscription_type=Periodic̏ꍇݒ肷B
   *
   * 6. L̏̂łG[ł΁AG[^[B
   *    ɏsꂽꍇRTC::RTC_OKŃ^[B
   *  
   * @since 0.4.0
   *
   * @else
   * @class DataOutPort
   * @brief OutPort abstruct class
   * @endif
   */
  class DataOutPort
    : public PortBase
  {
  public:
    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     *
     * @param name |[g
     * @param outport Yf[^o̓|[gɊ֘AtOutPortIuWFNg
     *                OutPortIuWFNgňf[^^Aobt@^Cvw肷
     * @param prop |[gݒpvpeB
     *
     * @else
     * @brief Constructor
     *
     * Constructor
     *
     * @param name Port name
     * @param outport OutPort object associated with this data output port.
     *                Specify also the data type and the buffer type used in
     *                the OutPort object.
     * @param prop Property for setting ports
     *
     * @endif
     */
    template <class DataType, template <class DataType> class Buffer>
    DataOutPort(const char* name, OutPort<DataType, Buffer>& outport,
		Properties& prop)
      : PortBase(name), m_outport(outport)
    {
      // PortProfile::properties ݒ
      addProperty("port.port_type", "DataOutPort");
      
      m_providers.push_back(new OutPortCorbaProvider<DataType>(outport));
      m_providers.back()->publishInterfaceProfile(m_profile.properties);
      
#ifdef RTC_SOCK_DATAPORT
      m_providers.push_back(new OutPortTcpSockProvider<DataType>(outport));
      m_providers.back()->publishInterfaceProfile(m_profile.properties);
#endif      
      m_consumers.push_back(new InPortCorbaConsumer<DataType>(outport));
#ifdef RTC_SOCK_DATAPORT
      m_consumers.push_back(new InPortTcpSockConsumer<DataType>(outport, prop));
#endif
    }
    
    /*!
     * @if jp
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~DataOutPort();
    
  protected:
    /*!
     * @if jp
     *
     * @brief Interface J
     *
     * ̃Iy[V́Anotify_connect() V[PX̎n߂ɃR[
     * 鏃z֐łB
     * notify_connect() ł́A
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile()
     *
     * ̏ protected ֐R[ڑsB
     * <br>
     * ̃Iy[V́AVK connector_id ɑ΂Ă͐ڑ̐A
     *  connector_id ɑ΂Ă͍XVK؂ɍsKvB
     *
     * @param connector_profile ڑɊւvt@C
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Publish interface information
     *
     * This operation is pure virutal function that would be called at the
     * beginning of the notify_connect() process sequence.
     * In the notify_connect(), the following methods would be called in order.
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile() 
     *
     * This operation should create the new connection for the new
     * connector_id, and should update the connection for the existing
     * connection_id.
     *
     * @param connector_profile The connection profile information
     *
     * @return The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t
    publishInterfaces(ConnectorProfile& connector_profile);
    
    /*! @if jp
     *
     * @brief Interface ɐڑ
     *
     * ̃Iy[V́Anotify_connect() V[PX̒ԂɃR[
     * 鏃z֐łB
     * notify_connect() ł́A
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile()
     *
     * ̏ protected ֐R[ڑsB
     *
     * @param connector_profile ڑɊւvt@C
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Subscribe to the interface
     *
     * This operation is pure virutal function that would be called at the
     * middle of the notify_connect() process sequence.
     * In the notify_connect(), the following methods would be called in order.
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile()
     *
     * @param connector_profile The connection profile information
     *
     * @return The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t
    subscribeInterfaces(const ConnectorProfile& connector_profile);
    
    /*!
     * @if jp
     *
     * @brief Interface ̐ڑ
     *
     * ̃Iy[V́Anotify_disconnect() V[PX̏IɃR[
     * 鏃z֐łB
     * notify_disconnect() ł́A
     * - disconnectNext()
     * - unsubscribeInterfaces()
     * - eraseConnectorProfile()
     * ̏ protected ֐R[ڑsB
     *
     * @param connector_profile ڑɊւvt@C
     *
     * @else
     *
     * @brief Disconnect the interface connection
     *
     * This operation is pure virutal function that would be called at the
     * end of the notify_disconnect() process sequence.
     * In the notify_disconnect(), the following methods would be called.
     * - disconnectNext()
     * - unsubscribeInterfaces()
     * - eraseConnectorProfile() 
     *
     * @param connector_profile The profile information associated with 
     *                          the connection
     *
     * @endif
     */
    virtual void
    unsubscribeInterfaces(const ConnectorProfile& connector_profile);
    
    /*!
     * @if jp
     * @brief InterfaceJpFunctor
     * @else
     * @brief Functor to publish the interface
     * @endif
     */
    struct publish
    {
      publish(SDOPackage::NVList& prop) : m_prop(prop) {}
      void operator()(OutPortProvider* provider)
      {
	provider->publishInterface(m_prop);
      }
      SDOPackage::NVList& m_prop;
    };
    
    /*!
     * @if jp
     * @brief InterfaceڑpFunctor
     * @else
     * @brief Functor to unsubscribe the interface
     * @endif
     */
    struct unsubscribe
    {
      unsubscribe(const SDOPackage::NVList& prop) : m_prop(prop) {}
      void operator()(InPortConsumer* consumer)
      {
	consumer->unsubscribeInterface(m_prop);
      }
      const SDOPackage::NVList& m_prop;
    };
    
  private:
    std::vector<OutPortProvider*> m_providers;
    std::vector<InPortConsumer*> m_consumers;
    OutPortBase& m_outport;
    
    PublisherFactory m_pf;
    
    struct subscribe
    {
      subscribe(const ConnectorProfile& prof)
	: m_prof(&prof), _consumer(NULL) 
      {
      }
      
      subscribe(const subscribe& subs)
	: m_prof(subs.m_prof),
	  _consumer(subs._consumer)
      {
      }
      
      subscribe& operator=(const subscribe& subs)
      {
	if (this == &subs) return *this;
	m_prof = subs.m_prof;
	_consumer = subs._consumer;
	return *this;
      }
      
      void operator()(InPortConsumer* cons)
      {
	if (cons->subscribeInterface(m_prof->properties))
	  {
	    _consumer = cons;
	  }
      }
      const ConnectorProfile* m_prof;
      InPortConsumer* _consumer;
    };
  };
}; // namespace RTC

#endif // DataOutPort_h
