// -*- C++ -*-
/*!
 * @file PortAdmin.h
 * @brief RTC's Port administration class
 * @date $Date: 2007-12-31 03:08:05 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PortAdmin.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef PortAdmin_h
#define PortAdmin_h

#include <rtm/idl/RTCSkel.h>
#include <rtm/PortBase.h>
#include <rtm/ObjectManager.h>

namespace RTC
{
  /*!
   * @if jp
   * @class PortAdmin
   * @brief PortAdmin NX
   *
   * e Port ̊ǗsNXB
   * Port ̓o^/o^ȂǊeǗsƂƂɁAo^Ă Port
   * Port ̊ǗsNXB
   *
   * @since 0.4.0
   *
   * @else
   * @class PortAdmin
   * @brief PortAdmin class
   *
   * This is a class to manage various Ports.
   * It executes various management operations such as registering and 
   * unregistering Port etc and also manages registered ports.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class PortAdmin
  {
  public:
    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     *
     * @param orb ORB
     * @param poa POA
     *
     * @else
     * @brief Constructor
     *
     * Constructor
     *
     * @param orb ORB
     * @param poa POA
     *
     * @endif
     */
    PortAdmin(CORBA::ORB_ptr orb, PortableServer::POA_ptr poa);
    
    /*!
     * @if jp
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~PortAdmin(){};
    
    /*!
     * @if jp
     *
     * @brief Port Xg̎擾
     *
     * registerPort() ɂo^ꂽ Port  Xg擾B
     *
     * @return Port Xg
     *
     * @else
     *
     * @brief Get PortList
     *
     * This operation returns the pointer to the PortList of Ports registered
     * by registerPort().
     *
     * @return The pointer points PortList
     *
     * @endif
     */
    PortList* getPortList() const;
    
    /*!
     * @if jp
     *
     * @brief PorProfile Xg̎擾
     *
     * registerPort() ɂo^ꂽ Port  Profile Xg擾B
     *
     * @return PortProfile Xg
     *
     * @else
     *
     * @brief Get PorProfileList
     *
     * This operation gets the Profile list of Ports registered by 
     * registerPort().
     *
     * @return The pointer points PortProfile list
     *
     * @endif
     */
    PortProfileList getPortProfileList() const;
    
    /*!
     * @if jp
     *
     * @brief Port ̃IuWFNgQƂ̎擾
     *
     * port_name Ŏw肵 Port ̃IuWFNgQƂԂB
     * port_name Ŏw肷 Port ͂炩 registerPort() œo^Ă
     * Ȃ΂ȂȂB
     *
     * @param port_name QƂԂPort̖O
     *
     * @return Port_ptr Port̃IuWFNgQ
     *
     * @else
     *
     * @brief Get the reference to Port object
     *
     * This operation returns the reference of Port object specified
     * by port_name.
     * The port specified by port_name must be already registered in 
     * registerPort().
     *
     * @param port_name The name of Port to be returned the reference.
     *
     * @return Port_ptr Port object reference.
     *
     * @endif
     */
    Port_ptr getPortRef(const char* port_name) const;
    
    /*!
     * @if jp
     *
     * @brief Port ̃T[og̃|C^̎擾
     *
     * port_name Ŏw肵 Port ̃T[og̃|C^ԂB
     * port_name Ŏw肷 Port ͂炩 registerPort() œo^Ă
     * Ȃ΂ȂȂB
     *
     * @param port_name QƂԂPort̖O
     *
     * @return PortBase* PortT[ogNX̃|C^
     *
     * @else
     *
     * @brief Get pointer to the Port's servant
     *
     * This operation returns the pointer to the PortBase servant registered
     * by registerPort().
     * The port specified by port_name must be already registered in 
     * registerPort().
     *
     * @param port_name The name of Port to be returned the servant pointer.
     *
     * @return PortBase* The pointer to Port's servant.
     *
     * @endif
     */
    PortBase* getPort(const char* port_name) const;
    
    /*!
     * @if jp
     *
     * @brief Port o^
     *
     *  port Ŏw肳ꂽ Port ̃T[ogo^B
     * o^ꂽ Port ̃T[og̓RXgN^ŗ^ꂽPOA 
     * activate ÃIuWFNgQƂPortProfileɃZbgB
     *
     * @param port Port T[og
     *
     * @else
     *
     * @brief Regsiter the Port
     *
     * This operation registers the Port's servant given by argument.
     * The given Port's servant will be activated on the POA that is given
     * to the constructor, and the created object reference is set
     * to the Port's profile.
     *
     * @param port The Port's servant.
     *
     * @endif
     */
    void registerPort(PortBase& port);
    
    /*!
     * @if jp
     *
     * @brief Port ̓o^
     *
     *  port Ŏw肳ꂽ Port ̓o^B
     * 폜 Port  deactivate APortProfilẽt@Xɂ́A
     * nillB
     *
     * @param port Port T[og
     *
     * @else
     *
     * @brief Unregister the Port registration
     *
     * This operation unregisters the Port registration.
     * When the Port is unregistered, Port is deactivated, and the object
     * reference in the Port's profile is set to nil.
     *
     * @param port The Port's servant.
     *
     * @endif
     */
    void deletePort(PortBase& port);
    
    /*!
     * @if jp
     *
     * @brief ̎wɂPort ̓o^
     *
     * Ŏw肳ꂽO Port ̓o^폜B
     * 폜 Port  deactivate APortProfilẽt@Xɂ́A
     * nillB
     *
     * @param port_name Port ̖O
     *
     * @else
     *
     * @brief Unregister the Port's registration by its name
     *
     * This operation unregister the Port's registration specified by port_ name.
     * When the Port is unregistered, Port is deactivated, and the object
     * reference in the Port's profile is set to nil.
     *
     * @param port_name The Port's name.
     *
     * @endif
     */
    void deletePortByName(const char* port_name);
    
    /*!
     * @if jp
     *
     * @brief SĂ Port deactivateo^폜
     *
     * o^ĂSĂPortɑ΂āAT[ogdeactivatesA
     * o^Xg폜B
     *
     * @else
     *
     * @brief Deactivate all Ports and unregister them
     *
     * This operation deactivates the all Port and deletes the all Port's
     * registrations from the list.
     *
     * @endif
     */
    void finalizePorts();
    
  private:
    // ORB ւ̃|C^
    CORBA::ORB_var m_pORB;
    
    // POA ւ̃|C^
    PortableServer::POA_var m_pPOA;
    
    // Port̃IuWFNgt@X̃Xg. PortList
    PortList m_portRefs;
    
    
    template <class T>
    class comp_op
    {
    public:
      comp_op(const char* _name)
	: m_name(_name)
      {
      }
      comp_op(T* obj) 
	: m_name((const char*)(obj->getProfile().name))
      {
      }
      bool operator()(T* obj)
      {
	std::string name((const char*)obj->getProfile().name);
	return m_name == name;
      }
    private:
      std::string m_name;
    };
    
    class port_prof_collect
    {
    public:
      port_prof_collect(PortProfileList& p) : m_p(p) {}
      void operator()(const PortBase* port)
      {
	CORBA_SeqUtil::push_back(m_p, port->getPortProfile());
      }
    private:
      PortProfileList& m_p;
    };
    
    struct find_port_name;
    struct del_port;
    
    // T[og𒼐ڊi[IuWFNg}l[W
    ObjectManager<const char*, PortBase, comp_op<PortBase> > m_portServants;
    
  };
};     // namespace RTC
#endif // PortAdmin_h
