// -*- C++ -*-
/*!
 * @file RTCUtil.h
 * @brief RTComponent utils
 * @date $Date: 2007-12-31 03:08:06 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2007-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: RTCUtil.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef RTCUtil_h
#define RTCUtil_h

#include <rtm/idl/RTCSkel.h>
#include <rtm/idl/OpenRTMSkel.h>

/*!
 * @if jp
 * @namespace RTC
 *
 * @brief RTR|[lgp[eBeB֐
 *
 * RTR|[lgɑ΂Ĉȉ̃[eBeB֐񋟂B
 * 
 * - isDataFlowParticipant
 * - isFsmParticipant
 * - isFsmObject
 * - isMultiModeObject
 *
 * @else
 * @namespace RTC
 *
 * @brief Utility functions for RT-Component
 *
 * This provides the following utility functions to RT-Component.
 * 
 * - isDataFlowParticipant
 * - isFsmParticipant
 * - isFsmObject
 * - isMultiModeObject
 *
 * @endif
 */
namespace RTC_Utils
{
  /*!
   * @if jp
   *
   * @brief DataFlowComponent ł邩肷
   *
   * w肳ꂽRTR|[lg DataFlowComponent ł邩肷B
   * DataFlowComponent́A ExecutionContext  Semantics 
   * Periodic Sampled Data Processing ̏ꍇɗpRTR|[lǧ^
   * łB
   *
   * @param obj Ώۂ CORBA IuWFNg
   *
   * @return DataFlowComponent 茋
   *
   * @since 0.4.0
   *
   * @else
   *
   * @brief Confirm whether specified RT-Component is DataFlowComponent
   *
   * Confirm whether specified RT-Component is DataFlowComponent.
   * DataFlowComponent is a type of the RT-Component which is used 
   * when Semantics of ExecutionContext is Periodic Sampled Data Processing.
   *
   * @param obj The target CORBA object for the investigation
   *
   * @return Investigation result of DataFlowComponent
   *
   * @since 0.4.0
   *
   * @endif
   */
  bool isDataFlowComponent(CORBA::Object_ptr obj);
  
  /*!
   * @if jp
   *
   * @brief FsmParticipant ł邩肷
   *
   * w肳ꂽRTR|[lg FsmParticipant ł邩肷B
   * FsmParticipant ́A ExecutionContext  Semantics 
   * Stimulus Response Processing ̏ꍇɁAԓ̃ANV`邽߂
   * pRTR|[lǧ^łB
   *
   * @param obj Ώۂ CORBA IuWFNg
   *
   * @return FsmParticipant 茋
   *
   * @since 0.4.0
   *
   * @else
   *
   * @brief Confirm whether specified RT-Component is FsmParticipant
   *
   * Confirm whether specified RT-Component is FsmParticipant.
   * FsmParticipant is a type of the RT-Component which is used when Semantics
   * of ExecutionContext is Stimulus Response Processing. It is used to define 
   * the actions in the state.
   *
   * @param obj The target CORBA object for the investigation
   *
   * @return Investigation result of FsmParticipant
   *
   * @since 0.4.0
   *
   * @endif
   */
  bool isFsmParticipant(CORBA::Object_ptr obj);
  
  /*!
   * @if jp
   *
   * @brief Fsm ł邩肷
   *
   * w肳ꂽRTR|[lg Fsm ł邩肷B
   * Fsm ́A ExecutionContext  Semantics  Stimulus Response Processing 
   * ꍇɁAԑJڂ`邽߂ɗpRTR|[lǧ^łB
   *
   * @param obj Ώۂ CORBA IuWFNg
   *
   * @return Fsm 茋
   *
   * @since 0.4.0
   *
   * @else
   *
   * @brief Confirm whether specified RT-Component is Fsm
   *
   * Confirm whether specified RT-Component is Fsm.
   * Fsm is a type of the RT-Component that is used when Semantics of 
   * ExecutionContext is Stimulus Response Processing. It is uset to define the 
   * state transition.
   *
   * @param obj The target CORBA object for the investigation
   *
   * @return Investigation result of Fsm
   *
   * @since 0.4.0
   *
   * @endif
   */
  bool isFsmObject(CORBA::Object_ptr obj);
  
  /*!
   * @if jp
   *
   * @brief multiModeComponent ł邩肷
   *
   * w肳ꂽRTR|[lg multiModeComponent ł邩肷B
   * multiModeComponent ́A ExecutionContext  Semantics  Modes of Operation 
   * ̏ꍇɁA Mode `邽߂ɗpRTR|[lǧ^łB
   *
   * @param obj Ώۂ CORBA IuWFNg
   *
   * @return multiModeComponent 茋
   *
   * @since 0.4.0
   *
   * @else
   *
   * @brief Confirm whether specified RT-Component is multiModeComponent
   *
   * Confirm whether specified RT-Component is multiModeComponent.
   * multiModeComponent is a type of the RT-Component which is used when
   * Semantics of ExecutionContext is Modes of Operation. It is used to define
   * Mode.
   *
   * @param obj The target CORBA object for the investigation
   *
   * @return Investigation result of multiModeComponent
   *
   * @since 0.4.0
   *
   * @endif
   */
  bool isMultiModeObject(CORBA::Object_ptr obj);
}; // namespace RTC_Utils
#endif // RTCUtil_h
